/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <tizen.h>
#include <service_app.h>
#include "main.h"
#include <sync_adapter.h>
#include <stdlib.h>
#include <message_port.h>

int service_port_id = -1;

bool on_start_cb(account_h account, const char *sync_job_name, const char *sync_capability,
                 bundle *sync_job_user_data)
{
    dlog_print(DLOG_DEBUG, LOG_TAG,
               "*************************** on_start_cb ***************************");
    int val = -1;
    int ret = bundle_get_byte(sync_job_user_data, "str", (void**)&val, NULL);
    if (ret != BUNDLE_ERROR_NONE || val == -1) {
        dlog_print(DLOG_DEBUG, LOG_TAG, "bundle_get_str() failed : %s", get_error_message(ret));
        return true;
    }
    dlog_print(DLOG_DEBUG, LOG_TAG, "Received string: %d", val);
    char *msg_to_send = NULL;
    if (val == ON_DEMAND_JOB_DONE)
        msg_to_send = "Service >>> Sync on demand is done";
    else if (val == PERIODIC_JOB_DONE)
        msg_to_send = "Service >>> Periodic sync is done";
    else if (val == SYNC_JOB_NOW)
        msg_to_send = "Service >>> Calendar data are synced now";

    bool exists = false;
    ret = message_port_check_trusted_remote_port(SYNC_ADAPTER_APP_ID, REMOTE_PORT, &exists);
    if (ret == MESSAGE_PORT_ERROR_NONE && exists) {
        bundle *b = bundle_create();
        ret = get_last_result();
        if (ret != BUNDLE_ERROR_NONE) {
            dlog_print(DLOG_DEBUG, LOG_TAG, "bundle_create() failed : %s", get_error_message(ret));
            return true;
        }
        ret = bundle_add_str(b, "message", msg_to_send);
        if (b == NULL || ret != BUNDLE_ERROR_NONE) {
            dlog_print(DLOG_DEBUG, LOG_TAG, "bundle_add_str() failed : %s", get_error_message(ret));
            return true;
        }
        ret = message_port_send_trusted_message(SYNC_ADAPTER_APP_ID, REMOTE_PORT, b);
        if (ret != MESSAGE_PORT_ERROR_NONE) {
            dlog_print(DLOG_DEBUG, LOG_TAG, "message_port_send_trusted_message() failed : %s", get_error_message(ret));
        }
        bundle_free(b);
    }
    return true;
}

void on_cancel_cb(account_h account, const char *sync_job_name, const char *sync_capability,
                  bundle *sync_job_user_data)
{
    dlog_print(DLOG_DEBUG, LOG_TAG,
               "*************************** on_cancel_cb **************************");
}

void on_message(int trusted_local_port_id, const char *remote_app_id, const char *remote_port,
                bool trusted_remote_port, bundle *message, void *user_data)
{
    dlog_print(DLOG_DEBUG, LOG_TAG,
               "***************************  on_message  **************************");

    char *val = NULL;
    bundle_get_str(message, "message", &val);
    dlog_print(DLOG_DEBUG, LOG_TAG, "Received string: %s", val);
    if (!strcmp(val, "exit"))
        service_app_exit();
}

bool service_app_create(void *data)
{
    int ret = sync_adapter_set_callbacks((sync_adapter_start_sync_cb) on_start_cb,
                                         (sync_adapter_cancel_sync_cb) on_cancel_cb);
    dlog_print(DLOG_DEBUG, LOG_TAG, "> > > callbacks set: %d", ret);
    service_port_id =
        message_port_register_trusted_local_port(SELF_PORT,
                (message_port_trusted_message_cb) on_message,
                NULL);

    bool exists = false;
    ret = message_port_check_trusted_remote_port(SYNC_ADAPTER_APP_ID, REMOTE_PORT, &exists);
    if (ret == MESSAGE_PORT_ERROR_NONE && exists) {
        bundle *b = bundle_create();
        ret = get_last_result();
        if (ret != BUNDLE_ERROR_NONE) {
            dlog_print(DLOG_DEBUG, LOG_TAG, "bundle_create() failed : %s", get_error_message(ret));
            return true;
        }

        ret = bundle_add_str(b, "message", "Service >>> Service has been started");
        if (b == NULL || ret != BUNDLE_ERROR_NONE) {
            dlog_print(DLOG_DEBUG, LOG_TAG, "bundle_add_str() failed : %s", get_error_message(ret));
            return true;
        }
        ret = message_port_send_trusted_message(SYNC_ADAPTER_APP_ID, REMOTE_PORT, b);
        if (ret != MESSAGE_PORT_ERROR_NONE) {
            dlog_print(DLOG_DEBUG, LOG_TAG, "message_port_send_trusted_message() failed : %s", get_error_message(ret));
        }
        bundle_free(b);
    }
    return true;
}

void service_app_terminate(void *data)
{
    sync_adapter_unset_callbacks();
    dlog_print(DLOG_DEBUG, LOG_TAG, "< < < callbacks unset.");
    return;
}

void service_app_control(app_control_h app_control, void *data)
{
    dlog_print(DLOG_DEBUG, LOG_TAG, "service_app_control called");

    char *app_id = NULL;
    app_control_get_app_id(app_control, &app_id);

    if (app_id != NULL) {
        dlog_print(DLOG_DEBUG, LOG_TAG, "caller %s", app_id);
        free(app_id);
    }

    return;
}

int main(int argc, char *argv[])
{
    service_app_lifecycle_callback_s event_callback;

    event_callback.create = service_app_create;
    event_callback.terminate = service_app_terminate;
    event_callback.app_control = service_app_control;

    return service_app_main(argc, argv, &event_callback, NULL);
}
